﻿using System;
using System.Collections.Generic;
using System.Collections.ObjectModel;
using System.Text;
using System.Windows;
using System.Windows.Media;
using System.Windows.Media.Imaging;
using System.Windows.Shapes;

using Microsoft.Maps.Core;
using MsdnMag.ObaLib;

namespace MsdnMag.ObaApp
{
	internal class ObaLayer : Layer
	{
		private ImageBrush _busImageBrush;
		private bool _isActive;
		private ObaPlugin _parent;
		private BusStopsPanel _resultPanel;
		private BusTripPanel _detailPanel;
		private Entity _currentHighlightEntity;
		private ObservableCollection<BusStopEntity> _busStopEntities;

		public bool IsActive
		{
			get { return _isActive; }
		}

		public ObaLayer(ObaPlugin parent)
			: base(parent.Token)
		{
			// prepare image brush to be used by layer icon
			_busImageBrush = new ImageBrush()
			{
				Stretch = Stretch.Fill,
				AlignmentX = AlignmentX.Center,
				AlignmentY = AlignmentY.Center,
				ImageSource = new BitmapImage(
					new Uri("/ObaApp;component/busicon.jpg", UriKind.Relative))
			};

			_parent = parent;

			_busStopEntities = new ObservableCollection<BusStopEntity>();
			_resultPanel = new BusStopsPanel(parent, this);
			_resultPanel.DataContext = _busStopEntities;

			this.Title = "Bus stops";
			this.Panel = _resultPanel;
			this.IconGenerator = LayerIconGenerator;
		}

		private UIElement LayerIconGenerator(LayerIconContext context)
		{
			var icon = new Rectangle
			{
				Width = 30,
				Height = 30,
				Fill = _busImageBrush
			};

			return icon;
		}

		protected override void BroughtToFront()
		{
			_isActive = true;
			_parent.RefreshBusStops();
		}

		protected override void NoLongerInFront()
		{
			_isActive = false;
			_parent.RefreshBusStops();
		}

		public void RefreshEntities(
			ICollection<BusStopEntity> addedEntities, ICollection<BusStopEntity> removedEntities)
		{
			foreach (BusStopEntity entity in removedEntities)
			{
				this.Entities.Remove(entity);
				_busStopEntities.Remove(entity);
			}

			foreach (BusStopEntity entity in addedEntities)
			{
				this.Entities.Add(entity);
				_busStopEntities.Add(entity);

				// register this entity to have popup behavior
				_parent.PopupContract.Register(entity, OnPopupStateChangeHandler);
			}
		}

		public void ClearEntities()
		{
			this.Entities.Clear();
			_busStopEntities.Clear();
		}

		private void OnPopupStateChangeHandler(PopupStateChangeContext context)
		{
			if (context.State == PopupState.Closed)
			{
				return;
			}

			BusStopEntity entity = (BusStopEntity)context.Entity;
			context.Title = entity.Name;
			context.Content = "Bus numbers: " + entity.BusRoutesAsString;

			if (context.State == PopupState.Normal)
			{
				// add Arrival times contributor
				context.Contributors.Add(new BusStopContributor(_parent, entity));

				Dictionary<string, object> parameter = new Dictionary<string, object>
				{
					{"Entity", entity}
				};
				var contributorHelper = _parent.ContributorHelper;

				// add Directions contributor
				context.Contributors.Add(contributorHelper.CreateDemandLoadContributor(
					"Microsoft/DirectionsContributorFactoryContract",
					parameter,
					"Directions"));

				// add Street side contributor
				context.Contributors.Add(contributorHelper.CreateAsyncContributor(
					"Microsoft/StreetsideContributorFactoryContract",
					parameter));
			}
		}

		public void ShowBusTrips(BusStopEntity entity, ICollection<BusTrip> busTrips)
		{
			if (_detailPanel == null)
			{
				_detailPanel = new BusTripPanel();
			}

			this.Title = "Bus stop: " + entity.Name;
			this.BackLinkText = "back";
			this.Panel = _detailPanel;
			_detailPanel.BusTripList.ItemsSource = busTrips;
			HighlightEntity(entity);
		}

		protected override void BackLinkClicked()
		{
			this.Title = "Bus stops";
			this.BackLinkText = null;
			this.Panel = _resultPanel;
			DeHighlightEntity();
		}

		public void HighlightEntity(Entity entity)
		{
			DeHighlightEntity();

			entity.Primitive.HighlightedState = HighlightedState.Highlighted;
			entity.Primitive.SelectedState = SelectedState.Selected;
			_currentHighlightEntity = entity;
		}

		public void DeHighlightEntity()
		{
			if (_currentHighlightEntity != null)
			{
				DeHighlightEntity(_currentHighlightEntity);
				_currentHighlightEntity = null;
			}
		}

		public void DeHighlightEntity(Entity entity)
		{
			if (entity != null && this.Entities.Contains(entity))
			{
				entity.Primitive.HighlightedState = HighlightedState.NotHighlighted;
				entity.Primitive.SelectedState = SelectedState.NotSelected;
			}

			_currentHighlightEntity = null;
		}
	}
}