﻿using System.Collections.Generic;
using System.Collections.ObjectModel;
using System.Diagnostics;
using System.Windows.Media;
using Microsoft.Maps.Core;
using Microsoft.Maps.MapControl;
using Microsoft.Maps.Plugins;
using MsdnMag.ObaLib;

namespace MsdnMag.ObaApp
{
	/// <summary>
	/// Plugin for the app
	/// </summary>
	public class ObaPlugin : Plugin
	{

		#region Imports

		[ImportSingle("Microsoft/LayerManagerContract", ImportLoadPolicy.Synchronous)]
		public LayerManagerContract LayerManager { get; set; }

		[ImportSingle("Microsoft/PopupContract", ImportLoadPolicy.Synchronous)]
		public PopupContract PopupContract { get; set; }

		[ImportSingle("Microsoft/ConfigurationContract", ImportLoadPolicy.Synchronous)]
		public ConfigurationContract ConfigurationContract { get; set; }

		[ImportSingle("Microsoft/MapContract", ImportLoadPolicy.Synchronous)]
		public MapContract Map { get; set; }

		[ImportSingle("Microsoft/PushpinFactoryContract", ImportLoadPolicy.Synchronous)]
		public PushpinFactoryContract PushpinFactory { get; set; }

		[ImportSingle("Microsoft/ContributorHelperContract", ImportLoadPolicy.Synchronous)]
		public ContributorHelperContract ContributorHelper { get; set; }

		#endregion

		private ObaLayer _layer;
		private ObaSearchHelper _searchHelper;
		private int _searchCount;
		private Dictionary<BusStop, int> _busStopCache = new Dictionary<BusStop, int>();

		/// <summary>
		/// Initializes a new instance of the <see cref="ObaPlugin"/> class.
		/// </summary>
		public ObaPlugin()
		{
		}

		/// <summary>
		/// When overridden in a derived class, the plug-in and its imports are initialized.
		/// </summary>
		public override void Initialize()
		{
			// obtain the application key from configuration file
			IDictionary<string, object> configs = ConfigurationContract.GetDictionary(this.Token);
			string appKey = (string)configs["ApplicationKey"];
			_searchHelper = new ObaSearchHelper(appKey);
			_searchHelper.SearchStopsByLocationCompleted += OnSearchBusStopsCompleted;
			_searchHelper.SearchArrivalTimesForStopCompleted += OnSearchArrialTimesComplete;

			// update the bus stop every time map view changes
			Map.TargetViewChangedThrottled += (s, e) => RefreshBusStops();
		}

		/// <summary>
		/// When overridden in a derived class, launches the plug-in with the specified parameters.
		/// </summary>
		/// <param name="activationParameters">The parameters to use when the plug-in is launched.</param>
		public override void Activate(IDictionary<string, string> activationParameters)
		{
			ShowResultLayer();
			RefreshBusStops();
		}

		private void ShowResultLayer()
		{
			if (_layer == null)
			{
				_layer = new ObaLayer(this);
			}

			if (LayerManager.ContainsLayer(_layer))
			{
				LayerManager.BringToFront(_layer);
			}
			else
			{
				LayerManager.AddLayer(_layer);
			}
		}

		private void OnMapRefresh(object sender, MapEventArgs e)
		{
			// refresh the bus stops whenever the map view changes
			RefreshBusStops();
		}

		internal void RefreshBusStops()
		{
			if (Map.ZoomLevel >= 14)
			{
				// search within the radius of 1km, maximum 150 results
				_searchHelper.SearchStopsByLocation(
					Map.Center.Latitude,
					Map.Center.Longitude,
					1000,
					150);
			}
			else
			{
				// clear all bus stop
				ClearBusStops();
			}
		}

		private void ClearBusStops()
		{
			_layer.ClearEntities();
		}

		private void OnSearchBusStopsCompleted(object sender, BusStopsEventArgs e)
		{
			_searchCount++;

			// filter out the new stops
			Collection<BusStopEntity> addedEntities = new Collection<BusStopEntity>();
			foreach (BusStop stop in e.BusStops)
			{
				if (!_busStopCache.ContainsKey(stop))
				{
					addedEntities.Add(new BusStopEntity(stop, PushpinFactory));
				}

				_busStopCache[stop] = _searchCount;
			}

			// filter out outgoing stops
			Collection<BusStopEntity> removedEntities = new Collection<BusStopEntity>();
			foreach (BusStopEntity bse in _layer.Entities)
			{
				if (_busStopCache[bse.BusStop] < _searchCount)
				{
					removedEntities.Add(bse);
					_busStopCache.Remove(bse.BusStop);
				}
			}

			_layer.RefreshEntities(addedEntities, removedEntities);
		}
		
		internal void SearchArrivalTimes(BusStopEntity _entity)
		{
			_searchHelper.SearchArrivalTimesForStop(_entity.StopId, _entity);
		}

		private void OnSearchArrialTimesComplete(object sender, BusTripsEventArgs e)
		{
			if (_layer.IsActive)
			{
				_layer.ShowBusTrips((BusStopEntity)e.UserState, e.BusTrips);
			}
		}
	}
}