﻿using System;

namespace MsdnMag.ObaLib
{
	/// <summary>
	/// Contains data about a particular bus trip
	/// </summary>
	public class BusTrip : IEquatable<BusTrip>
	{
		/// <summary>
		/// Gets the route id of this trip.
		/// </summary>
		/// <value>The route id.</value>
		public string RouteId { get; private set; }

		/// <summary>
		/// Gets route's name of this trip.
		/// </summary>
		/// <value>The name of the route.</value>
		public string RouteName { get; private set; }

		/// <summary>
		/// Gets the trip's id.
		/// </summary>
		/// <value>The trip id.</value>
		public string TripId { get; private set; }

		/// <summary>
		/// Gets the trip headsign.
		/// </summary>
		/// <remarks>
		/// This information is usually displayed in the LCD panel in front of the bus. 
		/// </remarks>
		/// <value>The trip headsign.</value>
		public string TripHeadsign { get; private set; }

		/// <summary>
		/// Gets the arrival time of this trip at a particular bus stop.
		/// </summary>
		/// <value>The arrival time.</value>
		public DateTime ArrivalTime { get; private set; }

		/// <summary>
		/// Initializes a new instance of the <see cref="BusTrip"/> class.
		/// </summary>
		/// <param name="routeId">The route id.</param>
		/// <param name="routeName">Name of the route.</param>
		/// <param name="tripId">The trip id.</param>
		/// <param name="tripHeadsign">The trip headsign.</param>
		/// <param name="predictedTime">The predicted time.</param>
		/// <param name="scheduledTime">The scheduled time.</param>
		public BusTrip(
			string routeId, 
			string routeName,
			string tripId, 
			string tripHeadsign, 
			long predictedTime,
			long scheduledTime)
		{
			this.RouteId = routeId;
			this.RouteName = routeName;
			this.TripId = tripId;
			this.TripHeadsign = tripHeadsign;

			// epoch = (DateTime.Now.ToUniversalTime().Ticks - 621355968000000000) / 10000000;

			long epoch = (predictedTime > 0) ? predictedTime : scheduledTime;
			long tick = epoch * 10000L + 621355968000000000L;

			DateTime UtcTime = new DateTime(tick, DateTimeKind.Utc);
			this.ArrivalTime = UtcTime.ToLocalTime();
		}

		/// <summary>
		/// Equalses the specified other.
		/// </summary>
		/// <param name="other">The other.</param>
		/// <returns></returns>
		public bool Equals(BusTrip other)
		{
			return other == null ? false : this.RouteId == other.RouteId;
		}

		/// <summary>
		/// Returns a hash code for this instance.
		/// </summary>
		/// <returns>
		/// A hash code for this instance, suitable for use in hashing algorithms and data structures like a hash table. 
		/// </returns>
		public override int GetHashCode()
		{
			return this.RouteId.GetHashCode();
		}

		/// <summary>
		/// Returns a <see cref="System.String"/> that represents this instance.
		/// </summary>
		/// <returns>
		/// A <see cref="System.String"/> that represents this instance.
		/// </returns>
		public override string ToString()
		{
			return this.TripHeadsign;
		}
	}
}