﻿using System;
using System.Collections.Generic;
using System.Collections.ObjectModel;
using System.Globalization;
using System.Json;
using System.Net;

namespace MsdnMag.ObaLib
{
	/// <summary>
	/// Contains the APIs to get data from the One Bus Away REST API
	/// </summary>
	public sealed class ObaSearchHelper
	{
		private string _obaApiBaseUri;
		private string _applicationKey;

		/// <summary>
		/// Initializes a new instance of the <see cref="ObaSearchHelper"/> class.
		/// </summary>
		/// <param name="applicationKey">The application key.</param>
		/// <param name="obaApiBaseUri">The base Url to OBA API server.</param>
		public ObaSearchHelper(string applicationKey, string obaApiBaseUri = "http://api.onebusaway.org")
		{
			if (obaApiBaseUri == null)
			{
				throw new ArgumentNullException("obaApiBaseUri");
			}

			if (applicationKey == null)
			{
				throw new ArgumentNullException("applicationKey");
			}

			_obaApiBaseUri = obaApiBaseUri;
			_applicationKey = applicationKey;
		}

		public event EventHandler<BusStopsEventArgs> SearchStopsByLocationCompleted;
		public event EventHandler<BusStopsEventArgs> SearchStopByIdCompleted;
		
		public event EventHandler<BusRoutesEventArgs> SearchRoutesByLocationCompleted;
		public event EventHandler<BusRoutesEventArgs> SearchRouteByIdCompleted;

		public event EventHandler<BusTripsEventArgs> SearchArrivalTimesForStopCompleted;

		private void OnSearchStopsByLocationCompleted(
			object sender, DownloadStringCompletedEventArgs e)
		{
			// if nobody has subscribed to the completed event, don't bother to parse the result
			if (SearchStopsByLocationCompleted == null)
			{
				return;
			}

			if (!e.Cancelled && e.Error == null)
			{
				string data = e.Result;
				JsonObject response = (JsonObject)JsonObject.Parse(data);

				int statusCode = (int)response["code"];
				if (statusCode == 200)
				{
					JsonObject stopsResult = (JsonObject)response["data"];
					JsonArray jsonStops = (JsonArray)stopsResult["stops"];

					Collection<BusStop> busStops = new Collection<BusStop>();
					foreach (JsonObject jsonStop in jsonStops)
					{
						busStops.Add(ParseBusStop(jsonStop));
					}

					// raise the Completed event
					if (SearchStopsByLocationCompleted != null)
					{
						var args = new BusStopsEventArgs(busStops);
						SearchStopsByLocationCompleted(this, args);
					}
				}
			}
		}

		public void SearchStopsByLocation(
			double latitude,
			double longitude,
			double radius,
			int maxResultCount)
		{
			// make sure maxCount has valid value
			maxResultCount = Math.Max(maxResultCount, 0);

			var parameters = new Dictionary<string, string>
				{
					{"lat", latitude.ToString()},
					{"lon", longitude.ToString()},
					{"maxCount", maxResultCount.ToString()}
				};
			
			if (IsValidValue(radius))
			{
				parameters.Add("radius", radius.ToString());
			}

			Uri uri = ConstructSearchUrl("api/where/stops-for-location.json", parameters);

			WebClient webClient = new WebClient();
			webClient.DownloadStringCompleted += OnSearchStopsByLocationCompleted;
			webClient.DownloadStringAsync(uri, null);
		}

		private void OnSearchRoutesByLocationCompleted(
			object sender, DownloadStringCompletedEventArgs e)
		{
			// if nobody has subscribed to the completed event, don't bother to parse the result
			if (SearchRoutesByLocationCompleted == null)
			{
				return;
			}

			if (!e.Cancelled && e.Error == null)
			{
				string data = e.Result;
				JsonObject response = (JsonObject)JsonObject.Parse(data);

				int statusCode = (int)response["code"];
				if (statusCode == 200)
				{
					JsonObject routesResult = (JsonObject)response["data"];
					JsonArray jsonRoutes = (JsonArray)routesResult["routes"];

					Collection<BusRoute> busRoutes = new Collection<BusRoute>();
					foreach (JsonObject jsonRoute in jsonRoutes)
					{
						busRoutes.Add(ParseBusRoute(jsonRoute));
					}

					// raise the Completed event
					if (SearchRoutesByLocationCompleted != null)
					{
						var args = new BusRoutesEventArgs(busRoutes);
						SearchRoutesByLocationCompleted(this, args);
					}
				}
			}
		}

		public void SearchRoutesByLocation(
			double latitude,
			double longitude,
			double radius)
		{
			var parameters = new Dictionary<string, string>
				{
					{"lat", latitude.ToString()},
					{"lon", longitude.ToString()}
				};

			if (IsValidValue(radius))
			{
				parameters.Add("radius", radius.ToString(CultureInfo.InvariantCulture));
			}

			Uri uri = ConstructSearchUrl("api/where/routes-for-location.json", parameters);
			
			WebClient webClient = new WebClient();
			webClient.DownloadStringCompleted += OnSearchRoutesByLocationCompleted;
			webClient.DownloadStringAsync(uri, null);
		}

		public void SearchRouteById(string routeId)
		{
			Uri uri = ConstructSearchUrl(
				string.Format("api/where/route/{0}.json", routeId),
				new Dictionary<string, string>());

			WebClient webClient = new WebClient();
			webClient.DownloadStringCompleted += OnSearchRouteByIdCompleted;
			webClient.DownloadStringAsync(uri, null);
		}

		private void OnSearchRouteByIdCompleted(object sender, DownloadStringCompletedEventArgs e)
		{
			if (SearchRouteByIdCompleted == null)
			{
				return;
			}

			if (!e.Cancelled && e.Error == null)
			{
				string data = e.Result;
				JsonObject response = (JsonObject)JsonObject.Parse(data);

				int statusCode = (int)response["code"];
				if (statusCode == 200)
				{
					JsonObject routeResult = (JsonObject)response["data"];
					JsonObject route = (JsonObject)routeResult["route"];

					BusRoute busRoute = ParseBusRoute(route);
					if (SearchRouteByIdCompleted != null)
					{
						var args = new BusRoutesEventArgs(new BusRoute[] { busRoute });
						SearchRouteByIdCompleted(this, args);
					}
				}
			}
		}

		public void SearchStopById(string stopId)
		{
			Uri uri = ConstructSearchUrl(
				string.Format("api/where/stop/{0}.json", stopId),
				new Dictionary<string, string>());

			WebClient webClient = new WebClient();
			webClient.DownloadStringCompleted += OnSearchStopByIdCompleted;
			webClient.DownloadStringAsync(uri, null);
		}

		private void OnSearchStopByIdCompleted(object sender, DownloadStringCompletedEventArgs e)
		{
			if (SearchStopByIdCompleted == null)
			{
				return;
			}

			if (!e.Cancelled && e.Error == null)
			{
				string data = e.Result;
				JsonObject response = (JsonObject)JsonObject.Parse(data);

				int statusCode = (int)response["code"];
				if (statusCode == 200)
				{
					JsonObject stopResult = (JsonObject)response["data"];
					JsonObject stop = (JsonObject)stopResult["stop"];

					BusStop busStop = ParseBusStop(stop);
					if (SearchStopByIdCompleted != null)
					{
						var args = new BusStopsEventArgs(new BusStop[] { busStop });
						SearchStopByIdCompleted(this, args);
					}
				}
			}
		}

		public void SearchArrivalTimesForStop(string stopId)
		{
			SearchArrivalTimesForStop(stopId, null);
		}

		public void SearchArrivalTimesForStop(string stopId, object userToken)
		{
			Uri uri = ConstructSearchUrl(
				string.Format("api/where/arrivals-and-departures-for-stop/{0}.json", stopId),
				new Dictionary<string, string>());

			WebClient webClient = new WebClient();
			webClient.DownloadStringCompleted += OnSearchArrivalTimesForStopCompleted;
			webClient.DownloadStringAsync(uri, userToken);
		}

		private void OnSearchArrivalTimesForStopCompleted(object sender, DownloadStringCompletedEventArgs e)
		{
			if (SearchArrivalTimesForStopCompleted == null)
			{
				return;
			}

			if (!e.Cancelled && e.Error == null)
			{
				string data = e.Result;
				JsonObject response = (JsonObject)JsonObject.Parse(data);

				int statusCode = (int)response["code"];
				if (statusCode == 200)
				{
					JsonObject stopResult = (JsonObject)response["data"];
					JsonArray tripResults = (JsonArray)stopResult["arrivalsAndDepartures"];

					Collection<BusTrip> alltrips = new Collection<BusTrip>();
					foreach (JsonObject jsonTrip in tripResults)
					{
						alltrips.Add(ParseBusTrip(jsonTrip));
					}

					if (SearchArrivalTimesForStopCompleted != null)
					{
						var args = new BusTripsEventArgs(alltrips, e.UserState);
						SearchArrivalTimesForStopCompleted(this, args);
					}
				}
			}
		}

		#region Helper functions

		private BusRoute ParseBusRoute(JsonObject route)
		{
			return new BusRoute(
				(string)route["id"],
				(string)route["shortName"],
				(string)route["longName"],
				(string)route["description"],
				null);
		}

		private BusStop ParseBusStop(JsonObject stop)
		{
			JsonValue direction;
			stop.TryGetValue("direction", out direction);

			Collection<BusRoute> routes = new Collection<BusRoute>();
			JsonArray jsonRoutes = (JsonArray)stop["routes"];
			foreach (var jr in jsonRoutes)
			{
				routes.Add(ParseBusRoute((JsonObject)jr));
			}

			return new BusStop(
				(string)stop["id"],
				(string)stop["name"],
				(double)stop["lat"],
				(double)stop["lon"],
				(string)direction,
				routes);
		}

		private BusTrip ParseBusTrip(JsonObject trip)
		{
			return new BusTrip(
				(string)trip["routeId"], 
				(string)trip["routeShortName"], 
				(string)trip["tripId"], 
				(string)trip["tripHeadsign"], 
				(long)trip["predictedArrivalTime"], 
				(long)trip["scheduledArrivalTime"]);
		}

		private Uri ConstructSearchUrl(string path, IDictionary<string, string> parameters)
		{
			parameters.Add("key", this._applicationKey);

			string query = string.Empty;
			foreach (var pair in parameters)
			{
				if (query.Length > 0)
				{
					query = query + "&";
				}
				query = query + string.Format("{0}={1}", pair.Key, pair.Value);
			}

			UriBuilder builder = new UriBuilder(_obaApiBaseUri)
			{
				Path = path,
				Query = query
			};

			return builder.Uri;
		}

		private bool IsValidValue(double x)
		{
			return x > 0 && !double.IsNaN(x) && !double.IsInfinity(x);
		}

		#endregion
	}
}